<?php 
if(!defined('ABSPATH'))
	die();

class IvyForm
{
	private static $identifier;
	
	private $form;
	private $valid = true;
	
	public $errors = array();

	
	public static function set_identifier( $identifier ) {
		self::$identifier = $identifier;
	}
	
	function __construct( $form = array() ) {
		add_action( self::$identifier . '-form-validate-required'     , array( $this, 'action_validate_required'   ), 1, 2 );
		add_action( self::$identifier . '-form-validate-date'         , array( $this, 'action_validate_date'       ), 1, 2 );
		add_action( self::$identifier . '-form-validate-time'         , array( $this, 'action_validate_time'       ), 1, 2 );
		add_action( self::$identifier . '-form-validate-current'      , array( $this, 'action_validate_current'    ), 1, 2 );
		add_action( self::$identifier . '-form-validate-email'        , array( $this, 'action_validate_email'      ), 1, 2 );
		add_action( self::$identifier . '-form-validate-phone'        , array( $this, 'action_validate_phone'      ), 1, 2 );
		add_action( self::$identifier . '-form-validate-match'        , array( $this, 'action_validate_match'      ), 1, 4 );
		add_action( self::$identifier . '-form-validate-maxlength'    , array( $this, 'action_validate_maxlength'  ), 1, 3 );
		add_action( self::$identifier . '-form-validate-numeric'      , array( $this, 'action_validate_numeric'    ), 1, 2 );
		add_action( self::$identifier . '-form-validate-zip'          , array( $this, 'action_validate_zip'        ), 1, 2 );
		add_action( self::$identifier . '-form-validate-username'     , array( $this, 'action_validate_username'   ), 1, 4 );
		add_action( self::$identifier . '-form-validate-account_email', array( $this, 'action_validate_account_email' ), 1, 4 );
		
		$this->set_rules( $form );
	}
	
	function set_rules( $rules ) {
		$this->form = $rules;
	}
	
	function action_validate_date( $field, $title ) {
		if (isset($_POST[$field]) && strptime( $_POST[$field], '%d-%m-%Y' ) === false ) 
			$this->set_error( $field, sprintf( ivy__( "The date specified in %s field is not valid. The correct format should be DD-MM-YYYY." ), $title ) );
	}

	function action_validate_time($field, $title)
	{
		if(isset($_POST[$field]) && strptime($_POST[$field], '%H:%M') === false)
			$this->set_error($field, sprintf(ivy__('The time specified in %s is not valid. The correct format should be HH:mm.'), $title));
	}
	
	function action_validate_current( $field, $title ) {
		$time = strtotime( $_POST[$field] );
		if ( $time && $time < mktime(0, 0, 0) ) {
			$this->set_error( $field, sprintf( ivy__( "The date specified in %s field must not be in the past." ), $title ) );
		}
	}
	
	function action_validate_required( $field, $title ) {
		if( empty($_POST[$field]) || (!is_array($_POST[$field]) && trim( $_POST[$field] ) == '') )
			$this->set_error( $field, sprintf( ivy__( "%s field is required." ), $title ) );
	}
	
	function action_validate_email( $field, $title ) {
		if ( ! empty( $_POST[$field] ) && is_email ( $_POST[$field] ) === false ) 
			$this->set_error( $field, sprintf( ivy__( "Please enter a valid %s." ), $title ) );
	}

	function action_validate_match( $field, $title, $matched_field, $match_title ) {
		if ( $_POST[$field] != $_POST[$matched_field] )
			$this->set_error( $field, sprintf( ivy__( '%1$s and %2$s fields do not match.' ), $title, $match_title ) );
	}

	function action_validate_maxlength( $field, $title, $maxlength ) {
		if ( strlen( $_POST[$field] ) > $maxlength ) {
			$this->set_error( $field, sprintf( ivy__( '%1$s field can only have a maximum of %2$d characters.' ), $title, $maxlength ) );
		}
	}

	function action_validate_phone( $field, $title ) {
		if ( ! empty( $_POST[$field] ) && preg_match( "/[^0-9\(\)\+-\s]/", $_POST[$field] ) )
			$this->set_error( $field, sprintf( ivy__( "The %s entered is not a valid phone number." ), $title ) );
	}

	function action_validate_numeric( $field, $title ) {
		if ( ! is_numeric_i18n( $_POST[$field] ) )
			$this->set_error( $field, sprintf( ivy__( "The %s field requires a valid number." ), $title ) );
	}
	
	function action_validate_username( $field, $title, $id, $error_callback ) {
		require_once( ABSPATH . WPINC . '/registration.php' );
		$username = $_POST[$field];
		
		if ( ! validate_username( $username ) ) {
			call_user_func_array( $error_callback, array( $field, sprintf( ivy__( "%s must only contain alphanumeric characters as well as dashes, dots and underscores." ), $title ) ) );
			return;
		}
		
		$search = username_exists( $username );
		if ( ! is_null( $search ) ) {
			if ( ( ! is_null( $id ) && $id != $search) || is_null( $id ) )
				call_user_func_array( $error_callback, array( $field, sprintf( ivy__( "The %s you chose is already taken by another user." ), $title ) ) );
		}
	}
	
	function action_validate_account_email( $field, $title, $id, $error_callback ) {
		require_once( ABSPATH . WPINC . '/registration.php' );
		$email = $_POST[$field];
		$search = email_exists( $email );

		if ( $search !== false ) {
			if ( ( ! is_null( $id ) && $id != $search) || is_null( $id ) )
				call_user_func_array( $error_callback, array( $field, sprintf( ivy__( "The %s you chose is already taken by another user." ), $title ) ) );
		}
	}
		
	function action_validate_zip( $field, $title ) {	
		$value = trim($_POST[$field]);
		if ( ! empty( $value ) ) {
			$countryRegs = array(
			'gb'      => '/\\A\\b[A-Z]{1,2}[0-9][A-Z0-9]? [0-9][ABD-HJLNP-UW-Z]{2}\\b\\z/i',
			'ca'      => '/\\A\\b[ABCEGHJKLMNPRSTVXY][0-9][A-Z][ ]?[0-9][A-Z][0-9]\\b\\z/i',
			'it'      => '/^[0-9]{5}$/i',
			'de'      => '/^[0-9]{5}$/i',
			'be'      => '/^[1-9]{1}[0-9]{3}$/i',
			'us'      => '/\\A\\b[0-9]{5}(?:-[0-9]{4})?\\b\\z/i',
			'nl'	  => '/^[0-9]{4}[ ]*[A-Z]{2}$/i',
			'default' => '/\\A\\b[0-9]{5}(?:-[0-9]{4})?\\b\\z/i' // Same as US.
			);

			// if no country fields are associated, "default" rule will be used
			if ( isset( $_POST['country'] ) && array_key_exists( strtolower( $_POST['country'] ), $countryRegs ) )
				$country = strtolower( $_POST['country'] );
			else
				$country = 'default';

			// adds hyphen if 9-digit zip code is used.
			if ( strlen( $value ) == 9 ) {
				$value = substr( $value, 0, 5 ) + '-' + substr( $value, -4 );
			}
			
			if ( ! preg_match( $countryRegs[$country], $value ) ) {
				$this->set_error( $field, "Please enter a valid {$title}.");
			}
		}
	}

	function value( $fieldname, $default = '', $mode = 'isset' ) {
		if ( isset( $_POST[$fieldname] ) && ( $mode == 'isset' || ( $mode == 'empty' && ! empty( $_POST[$fieldname] ) ) ) ) {
			return $_POST[$fieldname];
		}
		
		return $default;
	}

	function date_value( $fieldname, $default = '' ) {
		if ( isset( $_POST[$fieldname] ) ) {
			if ( is_numeric( $_POST[$fieldname] ) ) {
				return date( 'd-m-Y', $_POST[$fieldname] );
			}
			return $_POST[$fieldname];
		}
		return $default;
	}
	
	function selected( $fieldname, $value, $default = '' ) {
		$selected_value = isset( $_POST[$fieldname] ) ? $_POST[$fieldname] : $default;
		if ( $selected_value == $value )
			echo 'selected="selected"';
	}

	function checked( $fieldname, $value, $default = '') {
		if ( strpos( $fieldname, '[]' ) === false ) {
			$selected_value = isset( $_POST[$fieldname] ) ? $_POST[$fieldname] : $default;
		} else {
			$fieldname = str_replace( '[]', '', $fieldname );
			$selected_value = isset( $_POST[$fieldname] ) && in_array( $value, $_POST[$fieldname] ) ? $value : $default;
		}
		
		if ( $selected_value == $value )
			echo 'checked="checked"';
	}

	function field_html( $id, $properties ) {
		$output = '';
		
		$properties['field_id'] = $id;
		
		if ( $properties['type'] == 'select' ) {
			$properties['options'] = apply_filters( "albumregistry_form_select_options_{$id}", $properties['options'] );
		}
		
		$properties            = $this->prepare_titles( $properties );
		$properties            = $this->prepare_classes( $properties );
		$properties            = $this->prepare_ids( $properties );
		
		$output                = apply_filters( "albumregistry_form_{$properties['type']}", $output, $properties );
		
		return $output;
	}
	
	function display_errors( $mode = 'display' ) {
		$output = '';
		foreach ( $this->errors as $field => $message ) {
			$output .= "<li>{$message}</li>";
		}
		if ( $mode == 'return' )
			return $output;
		else 
			echo $output;
	}
	
	function state_dropdown( $name, $default_value = '' ) {
		$selected_value = ( isset( $_POST[$name] ) ) ? $_POST[$name] : $default_value;
		
		foreach ( $this->states as $name => $abbr ) {
			$selected = '';
			
			if ( $selected_value == $abbr ) 
				$selected = 'selected="selected"';
			
			if ( $abbr == "IT" ) {
				echo "<optgroup label='-----------------------'>";
				echo "<option {$selected} value='{$abbr}'>{$name}</option>";
				echo '</optgroup>';
			}
			else {
				echo "<option {$selected} value='{$abbr}'>{$name}</option>";
			}
		}
	}
	
	function country_dropdown( $name, $default_value = '' ) {
		$selected_value = ( isset( $_POST[$name] ) ) ? $_POST[$name] : $default_value;
		foreach ( $this->countries as $code => $name ) {
			$selected = '';
			if ( $selected_value == $code ) 
				$selected = 'selected="selected"';
		
			echo "<option {$selected} value='{$code}'>{$name}</option>";
		}
	}
	
	function validate() {
		$this->errors = array();
		
		if ( ! empty( $this->form ) ) {
			foreach ( $this->form as $field => $properties ) {
				if(!isset($properties['rules']))
					$properties['rules'] = array();
				else if ( ! is_array( $properties['rules'] ) )
					$properties['rules'] = explode( ',', str_replace( ' ', '', $properties['rules'] ) );

				foreach ( $properties['rules'] as $rule ) {
					if ( preg_match('/\[([^\]]+)\]/', $rule, $matches ) ) {
						$rule = str_replace( $matches[0], '', $rule );
						if ( isset( $properties['match_title'] ) )
							do_action( self::$identifier . "-form-validate-{$rule}", $field, $properties['title'], $matches[1], $properties['match_title'], array( $this, 'set_error' ) ); 
						else
							do_action( self::$identifier . "-form-validate-{$rule}", $field, $properties['title'], $matches[1], array( $this, 'set_error' ) );
					}
					elseif ( $rule == 'username' || $rule == 'account_email' ) {
						do_action( self::$identifier . "-form-validate-{$rule}", $field, $properties['title'], null, array( $this, 'set_error' ) );
					}
					else {
						do_action( self::$identifier . "-form-validate-{$rule}", $field, $properties['title'], array( $this, 'set_error' ) );
					}
					if ( $this->has_error( $field ) )
						break;
				}
			}
		}
		
		$this->valid = empty( $this->errors );
		return $this->valid;
	}
	
	public function is_valid() {
		return empty( $this->errors );
	}
			
	function has_error( $field ) {
		return ( isset( $this->errors[$field] ) && ! empty( $this->errors[$field] ) );
	}
	
	function set_error( $field, $message ) {
		$this->errors[$field] = $message;
	}
		
}